#!/usr/bin/python3.7
import os
import subprocess
import logging
import sys
from tree2gd.seq import read_fasta_file
import configparser
import subprocess
from multiprocessing import Pool



def mcl2fasta(minimal_taxa,outdir,step2out,step1out,step4out,args):
        logging.info("Reading mcl output file")
        clusterDICT = {}
        gene_idmap={}
        fa_list=[]
        count = 0
        if outdir[-1] != "/": outdir += "/"
        with open(os.sep.join([step1out,'all_sample2fa.list']),"rU") as idmap_file:
            for l in idmap_file:
                sp = l.strip().split('\t')
                gene_idmap[sp[0]]=sp[1]
        logging.info("gene_idmap read done.")
        with open(os.sep.join([step2out,'allmcl.out.OGs.group']),"rU") as infile:
                for line in infile:
                        spls=[]
                        if len(line) < 3: continue
                        genes = line.strip().split('\t')
                        for gene in genes:
                            spls.append(gene_idmap[gene])
                        if len(set(spls)) >= minimal_taxa:
                                count += 1
                                clusterID = str(count)
                                for seqID in genes:
                                        clusterDICT[seqID] = clusterID
        logging.info("clusters with at least "+str(minimal_taxa)+" taxa read")

        logging.info("Reading the fasta file "+os.sep.join([step1out,'all_sample.pep.faa']))

        for s in read_fasta_file(os.sep.join([step1out,'all_sample.pep.faa'])):
                seqid,seq = s.name,s.seq
                try:
                        clusterID = clusterDICT[seqid]
                        with open(outdir+"cluster"+clusterID+".fa","a") as outfile:
                            outfile.write(">"+seqid+"\n"+seq+"\n")
                            fa_list.append("cluster"+str(clusterID))
                except:
                        pass
        fa_list=list(set(fa_list))
        if args.cds2tree:
            logging.info("Choose to use cds sequence for gene tree construction and sort out the corresponding cds sequence files.")
            for s in read_fasta_file(os.sep.join([step4out,'all_cds.fa'])):
                    seqid,seq = s.name,s.seq
                    try:
                            clusterID = clusterDICT[seqid]
                            with open(outdir+"cluster"+clusterID+".cds","a") as outfile:
                                outfile.write(">"+seqid+"\n"+seq+"\n")
                    except:
                            pass
        return fa_list

def fa2tree(muscle,iqtree,fa,step4out,iqtreeop,args,Epal2nal):
    muscle_stdout=subprocess.run([muscle,'-in',os.sep.join([step4out,'all_fa',fa+".fa"]),'-out',os.sep.join([step4out,'muscle_out',fa+".pep.aln"]),],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    logging.debug('{} stdout:\n'.format(muscle) +muscle_stdout.stdout.decode('utf-8'))
    logging.debug('{} stderr:\n'.format(muscle) +muscle_stdout.stderr.decode('utf-8'))
    if args.cds2tree:
        Epal2nal_stdout=subprocess.run(Epal2nal+" "+os.sep.join([step4out,'muscle_out',fa+".pep.aln"])+" "+os.sep.join([step4out,'all_fa',fa+".cds"])+" -output fasta "+" > "+os.sep.join([step4out,'muscle_out',fa+".cds.aln"]),shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format(Epal2nal) +Epal2nal_stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format(Epal2nal) +Epal2nal_stdout.stderr.decode('utf-8'))
        iqtree_stdout=subprocess.run(iqtree+" -s "+os.sep.join([step4out,'muscle_out',fa+".cds.aln"])+" "+iqtreeop+" -pre "+os.sep.join([step4out,'iqtree_out',fa+".iqtree"]),shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format(iqtree) +iqtree_stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format(iqtree) +iqtree_stdout.stderr.decode('utf-8'))
    else:
        iqtree_stdout=subprocess.run(iqtree+" -s "+os.sep.join([step4out,'muscle_out',fa+".pep.aln"])+" "+iqtreeop+" -pre "+os.sep.join([step4out,'iqtree_out',fa+".iqtree"]),shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format(iqtree) +iqtree_stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format(iqtree) +iqtree_stdout.stderr.decode('utf-8'))
def run_fa2tree(args):
    fa2tree(args[0],args[1],args[2],args[3],args[4],args[5],args[6])

def run_tree2gd(step1out,args,config,step2out,step4out):
    logging.info("Writing step4 script files...")
    step4_sh=open(os.sep.join([step4out,'step4.sh']),"w")
    iqtreeop=''
    if not config.has_section('iqtree'):
        config.add_section('iqtree')
    if not config.has_option("iqtree", "-B"):
        config.set("iqtree", "-B","1000")
    if config.get("iqtree", "-B")=="0":
        config.remove_option("iqtree", "-B")
    if not config.has_option("iqtree", "-m"):
        if args.cds2tree:
            config.set("iqtree", "-m","HKY")
        else:
            config.set("iqtree", "-m","JTT+G4")
    for k,v in config.items("iqtree"):
        iqtreeop=' '.join((iqtreeop,' '.join((k,v))))
    tree2gd=config.get("software", "tree2gd")
    muscle=config.get("software", "muscle")
    iqtree=config.get("software", "iqtree")
    if args.cds2tree:
        Epal2nal=config.get("software", "Epal2nal")
    else:
        Epal2nal="Epal2nal.pl"
    step4_sh.writelines(["for F in "+os.sep.join([step4out,'all_fa/'])+"*.fa;do "+muscle+" -in $F -out "+os.sep.join([step4out,'muscle_out/'])+"`basename $F`.aln;done\n",
                            "for F in "+os.sep.join([step4out,'muscle_out/'])+"*.aln;do "+iqtree+iqtreeop+" -s $F -pre "+os.sep.join([step4out,'iqtree_out/'])+"`basename $F`.iqtree;done \n",
                            "ls "+os.sep.join([step4out,'iqtree_out/'])+"|grep contree > "+os.sep.join([step4out,'tree.list.tmp'])+"\n",
                            "cat -n "+os.sep.join([step4out,'tree.list.tmp'])+" > "+ os.sep.join([step4out,'tree.list'])+"\n",
                            tree2gd+" "+args.tree+" "+os.sep.join([step1out,'all_sample2fa.list'])+" "+os.sep.join([step4out,'tree.list'])+" "+os.sep.join([step4out,'Tree2GD_out/'])
                            ])
    step4_sh.close()
    if not args.only_script:
        if not os.path.exists(os.sep.join([step4out,'all_fa/'])):
            os.mkdir(os.sep.join([step4out,'all_fa/']))
        minimal_taxa=config.get("mcl2fasta", "min_taxa") if config.has_option("mcl2fasta", "min_taxa") and config.get("mcl2fasta", "min_taxa")!="" else '4'
        if int(minimal_taxa)<4:
            logging.warning("The mcl2fasta min_taxa cannot be set less than 4, otherwise a valid tree cannot be obtained ,it will be changed to the default 4!")
            minimal_taxa="4"
        fa_list=mcl2fasta(int(minimal_taxa),os.sep.join([step4out,'all_fa/']),step2out,step1out,step4out,args)
        logging.info("FASTA file selection has been completed ,according to MCL results.")
        try:
            subprocess.run(muscle,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("muscle can not run.")
            sys.exit(0)
        try:
            subprocess.run(iqtree,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("iqtree can not run.")
            sys.exit(0)
        try:
            subprocess.run(tree2gd,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("tree2gd can not run.")
            sys.exit(0)
        sh_list=[]
        if not os.path.exists(os.sep.join([step4out,'muscle_out/'])):
            os.mkdir(os.sep.join([step4out,'muscle_out/']))
        if not os.path.exists(os.sep.join([step4out,'iqtree_out/'])):
            os.mkdir(os.sep.join([step4out,'iqtree_out/']))
        for fa in fa_list:
            tmp=(muscle,iqtree,fa,step4out,iqtreeop,args,Epal2nal)
            sh_list.append(tmp)
        sh_pool=Pool(args.t)
        logging.info("Start runing step4 fa2tree.")
        sh_pool.map(run_fa2tree,sh_list)
        sh_pool.close()
        sh_pool.join()
        logging.info("step4 fa2tree has done.")
        logging.info("All sequences have completed the construction of the tree.")
        logging.info("Start WGD calculation.")
        tree2gdop=""
        if not config.has_section('tree2gd'):
            config.add_section('tree2gd')
        if not config.has_option("iqtree", "-B"):
            config.set("tree2gd", "--bp","0")
        for k,v in config.items("tree2gd"):
            tree2gdop=' '.join((tree2gdop,'='.join((k,v))))
        subprocess.run("ls "+os.sep.join([step4out,"iqtree_out"])+"|grep treefile |awk '{print \""+os.sep.join([step4out,"iqtree_out"])+"/\"$0;}' |cat -n > "+os.sep.join([step4out,'tree.list']),shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        tree2gd_out=subprocess.run(tree2gd+" "+args.tree+" "+os.sep.join([step1out,'all_sample2fa.list'])+" "+os.sep.join([step4out,'tree.list'])+" "+os.sep.join([step4out,'Tree2GD_out/'])+" "+tree2gdop,shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format(tree2gd) +tree2gd_out.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format(tree2gd) +tree2gd_out.stderr.decode('utf-8'))
        logging.info("step4 Tree2GD has done.")



#!/usr/bin/python3.7
import os
import subprocess
import logging
import sys
import configparser
import subprocess
from multiprocessing import Pool
import re

from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker


def run_plot(step6out,args,cf):
    logging.info("Start step6 summary plot...")
    os.chdir(step6out)
    if not args.only_script:
        stdout=subprocess.run('Rscript '+'Tree2GD_draw.R',shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        logging.info("R plot done.")
        logging.debug('{} stdout:\n'.format("Tree2GD_draw.R") +stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format("Tree2GD_draw.R") +stdout.stderr.decode('utf-8'))
        logging.info("Start html plot ...")
        if not os.path.exists(os.sep.join([step6out,"ks_html_plot"])):
            os.mkdir(os.sep.join([step6out,"ks_html_plot"]))
        file_names=[]
        file_names = os.listdir("html_plot_in")
        os.chdir(os.sep.join([step6out,"html_plot_in"]))
        sh_pool=Pool(args.t)
        sh_pool.map(html_plot,file_names)
        sh_pool.close()
        sh_pool.join()
        logging.info("ALL step6 plot has done.")

def html_plot(file):
    sp=file.split(".")
    with open(file) as f:
        l=f.readlines()
        i=0
        ks={}
        for line in l:
            if i==0:
                head=line.strip().split("\t")
                i=1
            else:
                t=line.strip().split("\t")
                name=t[0]
                del t[0]
                ks[name]=t
        bar=Bar(init_opts=opts.InitOpts(page_title=sp[0]+"-ks.htmlpolt",width="1500px",height="700px"))
        bar.add_xaxis(head)
        for key,value in ks.items():
            bar.add_yaxis(key,value,category_gap=0,stack="stack1",is_large=True)
        bar.set_series_opts(label_opts=opts.LabelOpts(is_show=False))
        bar.set_global_opts(
        title_opts=opts.TitleOpts(title=sp[0]+".ks.polt",subtitle="Plot by Tree2gd v1.0"),
        toolbox_opts=opts.ToolboxOpts(is_show=True,pos_top='10%',orient='vertical',pos_left='right',feature=opts.ToolBoxFeatureOpts(
        save_as_image=opts.ToolBoxFeatureSaveAsImageOpts(title="Save as png"),
        restore=opts.ToolBoxFeatureRestoreOpts(title="Recovery"),
        data_view=opts.ToolBoxFeatureDataViewOpts(title="Data",lang=['Data View','Close','Refresh']),
        data_zoom=opts.ToolBoxFeatureDataZoomOpts(is_show=False),
        magic_type=opts.ToolBoxFeatureMagicTypeOpts(line_title="Switch to line plot",bar_title="Switch to histogram plot",stack_title="Switch to stack plot",tiled_title="Switch to tiled plot"),
        brush=opts.ToolBoxFeatureBrushOpts(rect_title="Rectangle selection",polygon_title="Circle selection",line_x_title="X line selection",line_y_title="Y line selection",keep_title="Keep selection",clear_title="Clear selection"),
        )),
        legend_opts=opts.LegendOpts(pos_left = 'right',),
        datazoom_opts=[opts.DataZoomOpts(range_start=0,range_end=60),opts.DataZoomOpts(type_="inside")],
        xaxis_opts=opts.AxisOpts(name="Ks"),)
        bar.render(os.sep.join(["..","ks_html_plot",sp[0]+".ks.plot.html"]))



#!/usr/bin/python3.7

import os
import subprocess
import logging
import sys

def run_dolloparsimony(step1out,step2out,step3out,dolloparsimony,args):
    logging.info("Writing step3 script files...")
    step3_sh=open(os.sep.join([step3out,'step3.sh']),"w")
    step3_sh.write(' '.join([dolloparsimony,args.tree,os.sep.join([step1out,'all_sample2fa.list']),os.sep.join([step2out,'allmcl.out.OGs.group']),os.sep.join([step3out,'dollop.out'])+"\n"]))
    step3_sh.write("less "+os.sep.join([step3out,'dollop.out','summary.tree'])+"|cut -d '[' -f2|cut -d ']' -f1  >"+os.sep.join([step3out,'dollop.out.scv']))
    step3_sh.close()

    if not args.only_script:
        logging.info("Start runing step3 dollop: %s.."%(os.sep.join([step3out,'step3.sh'])))
        stdout=subprocess.run(['sh',os.sep.join([step3out,'step3.sh'])],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format('step2.sh') +stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format('step2.sh') +stdout.stderr.decode('utf-8'))
        logging.info("step3 dollop has done.")



#!/usr/bin/python3.7
import os
import subprocess
import logging
import sys
from tree2gd.seq import read_fasta_file
import configparser
import subprocess
from multiprocessing import Pool
import re

def run_kaks(sp_list,step1out,args,config,step4out,step5out,gene_pairs_idmap):
    logging.info("Writing step5 script files...")
    op=''
    if not config.has_section('KaKs_Calculator'):
        config.add_section('KaKs_Calculator')
    if not config.has_option("KaKs_Calculator", "-m"):
        config.set("KaKs_Calculator", "-m","YN")
    for k,v in config.items("KaKs_Calculator"):
        op=' '.join((op,' '.join((k,v))))
    step5_sh=open(os.sep.join([step5out,'step5.sh']),"w")
    KaKs_Calculator=config.get("software", "KaKs_Calculator")
    Kaks_aligncmd=config.get("software", "muscle")
    Epal2nal=config.get("software", "Epal2nal")
    step5_sh.writelines([   "perl ParaAT.pl -kaks "+KaKs_Calculator+op+" -h "+os.sep.join([step5out,"all_gene_pairs.list"])
                            +" -n "+os.sep.join([step5out,"all_cds.fa"])+" -a "+os.sep.join([step5out,'all_pep.fa'])+" -p "+str(args.t)+" -k -f axt -o "+os.sep.join([step5out,"all_gene_pairs_kaks"])+"\n",
                            "cd "+os.sep.join([step5out,"all_gene_pairs_kaks"])+"\n",
                            "for i in `ls *.axt`;do axt2one-line.py $i ${i}.one-line;done\n",
                            "ls *.one-line|while read id;do calculate_4DTV_correction.pl $id >${id%%one-line}4dtv;done\n"
                            "for i in `ls *.4dtv`;do awk 'NR>1{print $1\"\t\"$2}' $i >>all-4dtv.txt;done\n",
                            "for i in `ls *.kaks`;do awk 'NR>1{print $1\"\t\"$3\"\t\"$4\"\t\"$5}' $i >>all-kaks.txt;done\n",
                            "sort all-4dtv.txt|uniq >all-4dtv.results\n",
                            "sort all-kaks.txt|uniq >all-kaks.results\n",
                            "join -1 1 -2 1 all-kaks.results all-4dtv.results >all-results.txt\n",
                            "sed -i '1i\Seq\tKa\tKs\tKa/Ks\t4dtv_corrected' all-results.txt\n"
                            ])
    step5_sh.close()
    if not args.only_script:
        logging.info("Start KaKs calculation.")
        pepmap={}
        cdsmap={}
        logging.info("Readfile")
        for s in read_fasta_file(os.sep.join([step5out,'all_pep.fa'])):
                pepmap[s.name]=s.seq
        for s in read_fasta_file(os.sep.join([step5out,'all_cds.fa'])):
                cdsmap[s.name]=s.seq
        if not os.path.exists(os.sep.join([step5out,"sp_kaks_out"])):
            os.mkdir(os.sep.join([step5out,"sp_kaks_out"]))
        if not os.path.exists(os.sep.join([step5out,"all_gene_pairs_kaks"])):
            os.mkdir(os.sep.join([step5out,"all_gene_pairs_kaks"]))
        logging.info("Make dir")
        os.chdir(os.sep.join([step5out,"all_gene_pairs_kaks"]))
        logging.info("Start gene pairs kaks calculation.")
        a=0
        pairs_num=len(gene_pairs_idmap)
        b=pairs_num//20
        arg_list=[]

        for pair in gene_pairs_idmap:
            tmp=(pair,pepmap,cdsmap,KaKs_Calculator,Kaks_aligncmd,Epal2nal,op)
            arg_list.append(tmp)
        sh_pool=Pool(args.t)
        sh_pool.map(run_ks,arg_list)
            #a=a+1
            #sh_pool.apply_async(sub_sh,args=(pair,pepmap,cdsmap,KaKs_Calculator,Kaks_aligncmd,Epal2nal,op),callback=write_result)
            #if(a%b==0):
            #    logging.info("%s \%(%s pairs) completed."%((a//b)*5)%(a))
        sh_pool.close()
        sh_pool.join()
        logging.info("Gene pairs kaks calculation finish.")
        logging.info("Start summarize the Ks results of each species.")
        for pair in gene_pairs_idmap:
            out=sum_Ks(pair)
            write_result(out)
            a=a+1
            if(a%b==0):
                logging.info("%d %% (%d pairs) completed."%((a//b)*5,a))

        logging.info("ALL step5 kaks has done.")

def run_ks(args):
    sub_sh(args[0],args[1],args[2],args[3],args[4],args[5],args[6])

def sum_Ks(pair):
    out=[]
    with open(pair[2]+"-"+pair[3]+".cds_aln.axt.kaks") as kaks_file,open(pair[2]+"-"+pair[3]+".4dtv") as DTV_file:
        kaks=kaks_file.readlines()[1].split("\t")
        dtv=DTV_file.readlines()[1].split("\t")
        out=[pair[1],pair[0],pair[2],pair[3],kaks[2],kaks[3],kaks[4],dtv[1]]
    return out

def sub_sh(pair,pepmap,cdsmap,KaKs_Calculator,Kaks_aligncmd,Epal2nal,op):
    out=[]
    home_dir=os.path.dirname(os.path.abspath(__file__))
    try:
        pepseq1 = pepmap[pair[2]]
        pepseq2 = pepmap[pair[3]]
        cdsseq1 = cdsmap[pair[2]]
        cdsseq2 = cdsmap[pair[3]]
        with open(pair[2]+"-"+pair[3]+".pep","a") as outfile:
            outfile.write(">"+pair[2]+"\n"+pepseq1+"\n")
            outfile.write(">"+pair[3]+"\n"+pepseq2+"\n")
        with open(pair[2]+"-"+pair[3]+".cds","a") as outfile:
            outfile.write(">"+pair[2]+"\n"+cdsseq1+"\n")
            outfile.write(">"+pair[3]+"\n"+cdsseq2+"\n")

    except:
        logging.info("err")
        pass
    stdout=subprocess.run(Kaks_aligncmd+" -quiet  -in "+pair[2]+"-"+pair[3]+".pep  -out "+pair[2]+"-"+pair[3]+".pep_aln >> msg.msa",shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    Epal2nal_stdout=subprocess.run(Epal2nal+" "+pair[2]+"-"+pair[3]+".pep_aln "+pair[2]+"-"+pair[3]+".cds -output fasta -nogap -nomismatch > "+pair[2]+"-"+pair[3]+".cds_aln",shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    Fasta2AXT(pair[2]+"-"+pair[3]+".cds_aln",pair[2]+"-"+pair[3]+".cds_aln.axt")
    kaks_stdout=subprocess.run(KaKs_Calculator+op+" -i "+pair[2]+"-"+pair[3]+".cds_aln.axt -o  "+pair[2]+"-"+pair[3]+".cds_aln.axt.kaks >> msg.kaks",shell=True,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    axt2oneline((pair[2]+"-"+pair[3]+".cds_aln.axt"),(pair[2]+"-"+pair[3]+".one-line"))
    stdout=subprocess.run("perl "+os.sep.join([home_dir,"software","calculate_4DTV_correction.pl"])+" "+pair[2]+"-"+pair[3]+".one-line"+" > "+pair[2]+"-"+pair[3]+".4dtv",shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)


def write_result(input):
    with open(os.sep.join(["..","sp_kaks_out",input[0]+".kaks_4dtv.result"]),"a+") as out:
        out.write("\t".join([input[0],input[1],input[2],input[3],input[4],input[5],input[6],input[7]])+"\n")


def Fasta2AXT(input,output):
    id=""
    seq=""
    for s in read_fasta_file(input):
        if id=="":
            id=s.name
        else:
            id=id+"-"+s.name
        if seq=="":
            seq=s.seq
        else:
            seq=seq+'\n'+s.seq
    with open(output,"w") as outfile:
        outfile.write(id+"\n"+seq+"\n")

def axt2oneline(reader,out):#copyed
    mydict = {}
    writer = open(out,"w")
    with open(reader) as fh:
            for line in fh:
                    if re.search(r"\d+",line):
                        header = line.strip()
                        mydict[header] = []
                    else:
                        mydict[header].append(line.strip())
    for key,value in mydict.items():
        seqs = "".join(value)
        length = len(seqs)
        mid = int(length/2)
        seq1 = seqs[0:mid]
        seq2 = seqs[mid:]
        writer.write(key + "\n" + "\n".join([seq1,seq2]))
    writer.close()



#!/usr/bin/python3.7

import os
import subprocess
import logging
from multiprocessing import Pool
import sys
import configparser

def sub_sh(sh):
    logging.info("Start runing step1 blastp: %s.."%sh)
    stdout=subprocess.run(['sh',sh],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    logging.debug('{} stdout:\n'.format(sh) +stdout.stdout.decode('utf-8'))
    logging.debug('{} stderr:\n'.format(sh) +stdout.stderr.decode('utf-8'))
    logging.info("step1 blastp: %s has done."%sh)


def all2all_diamond(sp_list,config,out,diamond,seqkit,pep_postfix,args):
    logging.info("Writing step1 script files...")
    sp_num=len(sp_list)
    op=''
    sh_list=[]
    p=config.get("diamond","-p") if config.has_option("diamond","-p") and config.get("diamond","-p")!="" else args.t
    if int(p)>args.t:
        logging.warning("The number of diamond threads %s is greater than the total program thread %s, it will be set to %s."%(str(p),str(args.t),str(args.t)))
        p=args.t
    if not config.has_section('diamond'):
        config.add_section('diamond')
    config.set("diamond","-p",str(p))
    options=config.items("diamond")
    summary_sh = open(os.sep.join([out,'summary.sh']),"w")
    step1_sh = open(os.sep.join([out,'step1.sh']),"w")
    for k,v in options:
        op=' '.join((op,' '.join((k,v))))
    for i in sp_list:
        SH = open(os.sep.join([out,'sample_sh',i+".sh"]),"w")
        SH.write(diamond + ' makedb -d '+os.sep.join([args.i,i+pep_postfix])+' --in '+os.sep.join([args.i,i+pep_postfix])+"\n")
        for j in sp_list:
            SH.write(diamond+' blastp -f 6 --more-sensitive '+op+' -d '+os.sep.join([args.i,i+pep_postfix])+' -q '+os.sep.join([args.i,j+pep_postfix])+' -o '+os.sep.join([out,'allsample_blast',i+"-"+j+".blast\n"]))
            summary_sh.write("cat "+os.sep.join([out,'allsample_blast',i+"-"+j+".blast"])+" >> "+os.sep.join([out,'all_blastp.out'])+"\n")
        summary_sh.writelines(["cat "+os.sep.join([args.i,i+pep_postfix])+ " >> " +os.sep.join([out,'all_sample.pep.faa'])+"\n",
                        seqkit+" fx2tab -l " + os.sep.join([args.i,i+pep_postfix])+"|awk \'{print $1\"\\t\"$NF}\' >> "+os.sep.join([out,'all_sample.faa.length'])+"\n",
                        seqkit+" fx2tab -l " + os.sep.join([args.i,i+pep_postfix])+"|awk \'{print $1\"\\t"+i+"\"}\' >> "+os.sep.join([out,'all_sample2fa.list'])+"\n"])
        SH.close()
        sh_list.append(os.sep.join([out,'sample_sh',i+".sh"]))
    summary_sh.close()

    if not args.only_script:
        try:
            subprocess.run(diamond,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("diamond can not run.")
            sys.exit(0)
        try:
            subprocess.run(seqkit,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("seqkit can not run.")
            sys.exit(0)
        sh_pool=Pool(args.t//int(p))
        sh_pool.map(sub_sh,sh_list)
        sh_pool.close()
        sh_pool.join()
        logging.info("ALL step1 blastp has done.")
        logging.info("Start sorting results..")
        stdout=subprocess.run(['sh',os.sep.join([out,'summary.sh'])],stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    step1_sh.writelines("\n".join(sh_list))
    step1_sh.write(os.sep.join([out,'summary.sh']))
    step1_sh.close()



offset =33 # 33 for most newer reads, 64 for old Illumina reads

class Sequence:
	def __init__(self,name="",seq=""):
		self.name = name
		self.seq = seq
		self.qualstr = "" #string of quality scores in characters
		self.qualarr = [] #list of quality scores in ASCII numbers

	#input a string of quality characters, set both qualstr and qualarr
	def set_qualstr(self,qual):
		self.qualstr = qual
		if len(self.qualarr) == 0:
			for j in self.qualstr:
				quality_score = ord(j) - offset
				self.qualarr.append(quality_score)
				assert quality_score <= 41 and quality_score > 0, \
					"Change the offset in seq.py\nqual"

	#output in four-line fastq format
	def get_fastq(self):
		retstr = "@"+self.name+"\n"
		retstr += self.seq+"\n"
		retstr += "+\n"
		retstr += self.qualstr+"\n"
		return retstr

	def get_fasta(self):
		return ">"+self.name+"\n"+self.seq+"\n"

	def rev_comp(self):
		tseq = ""
		for i in self.seq[::-1]:
			if i.lower() == "a":
				tseq += "t"
			elif i.lower() == "t":
				tseq += "a"
			elif i.lower() == "c":
				tseq += "g"
			elif i.lower() == "g":
				tseq += "c"
			else:
				tseq += i
		self.seq = tseq

def fastq_generator(infile):
	line = infile.readline()
	while len(line) > 0: #no empty string at end-of-file in readline
		if line[0] == "@":
			name = line[1:].strip() #name of sequence, minus "@"
			seq = infile.readline().strip() #the actual sequence
			line = infile.readline().strip() #"+"
			qual = infile.readline().strip() #the quality string
			tseq = Sequence(name=name,seq=seq)
			tseq.set_qualstr(qual)
			yield tseq
		line = infile.readline()



def read_fasta_file(filename):
	"""given the path to a fasta file, retern a list of seq objects"""
	fl = open(filename,"r")
	seqlist = [] #list of sequence objects
	templab = ""
	tempseq = ""
	first = True
	for i in fl:
		if i[0] == ">":
			if first == True:
				first = False
			else:#if not first store lastseq read
				seqlist.append(Sequence(templab,tempseq))
			templab = i.strip()[1:].split(" ")[0]
			tempseq = ""
		else:
			tempseq = tempseq + i.strip()
	fl.close()
	seqlist.append(Sequence(templab,tempseq))
	return seqlist



#!/usr/bin/python3.7
import os
import subprocess
import logging
import sys

def run_phymcl(step1out,phymcl,args,options,step2out):
    logging.info("Writing step2 script files...")
    op=''
    for k,v in options:
        op=' '.join((op,' '.join((k,v))))
    step2_sh=open(os.sep.join([step2out,'step2.sh']),"w")
    step2_sh.write(phymcl+ " -in " +os.sep.join([step1out,'all_blastp.out'])+" -threads "+ str(args.t) + " -length " + os.sep.join([step1out,'all_sample.faa.length']) +
                        " -species " + os.sep.join([step1out,'all_sample2fa.list']) + " -tree " + os.sep.join([step2out,"phymcl.input.tree"]) + " -out " + os.sep.join([step2out,'allmcl.out'])+"\n")
    step2_sh.close()
    if not args.only_script:
        try:
            subprocess.run(phymcl,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        except FileNotFoundError:
            logging.error("phymcl can not run.")
            sys.exit(0)
        logging.info("Start runing step2 MCL: %s.."%(os.sep.join([step2out,'step2.sh'])))
        stdout=subprocess.run(['sh',os.sep.join([step2out,'step2.sh'])],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logging.debug('{} stdout:\n'.format('step2.sh') +stdout.stdout.decode('utf-8'))
        logging.debug('{} stderr:\n'.format('step2.sh') +stdout.stderr.decode('utf-8'))
        logging.info("step2 mcl has done.")



#!/usr/bin/python3.5

try:
    from setuptools import setup
    from setuptools import Command
    from setuptools import Extension
except ImportError:
    sys.exit(
        "We need the Python library setuptools to be installed. "
        "Try runnning: python -m ensurepip"
    )

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='Tree2gd',
    version='1.0.38',
    packages=['tree2gd'],
    license='GPL',
    long_description_content_type='text/x-rst',
    long_description=long_description,
    author='Duoyuan chen',
    author_email='18110700097@fudan.edu.cn',
    description='Tree2gd',
    py_modules=['tree2gd_main'],
    include_package_data=True,
    url='https://github.com/Dee-chen/Tree2gd',
    install_requires=[
        'configparser',
        'matplotlib',
        'argparse',
        'coloredlogs',
        'pyecharts',
        'bio'
    ],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        Tree2gd=tree2gd_main:main
        Tree2gd_test=tree2gd_main:test
    ''',
)



#!/usr/bin/python3.7

import argparse
#import threading
import time
import os
import configparser
import logging
import coloredlogs
import sys
import subprocess
import multiprocessing
import shutil
from Bio import Phylo
from tree2gd.seq import read_fasta_file
import re

class NewConfigParser(configparser.ConfigParser):
    def optionxform(self, optionstr):
        return optionstr

def err_exit():
    sys.exit('\033[1;31;47m!!The Tree2gd program exited abnormally, please check the log file !!\033[0m')

def cmd_check(cmd):
    try:
        p=subprocess.run(cmd,check=True,stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
        logging.debug(p.stdout.decode("utf-8").strip())
        pass
    except FileNotFoundError:
        logging.error("The program %s path exists, please check the settings"%(cmd))
        err_exit()
    except subprocess.CalledProcessError:
        logging.error("The program %s not working properly, please check the settings"%(cmd))
        err_exit()
    logging.info("The program %s working properly [OK]"%(cmd))

def main():
    pwd= os.getcwd()
    parser = argparse.ArgumentParser(description='Tree2GD:A pipeline for WGD V1.0.38')
    parser.add_argument('-i', type=str, required=True, metavar='input_dir',help='The CDS and pep dir.')
    parser.add_argument('-tree', type=str, required=True, metavar='phytree.nwk',help='The phytree file.')
    parser.add_argument('-t', type=int,metavar='t',default='1', help='Thread num.default:1')
    parser.add_argument('-o', type=str,metavar='outputdir',default=os.sep.join([pwd,'output']),help='The output dir.default:'+os.sep.join([pwd,'output']))
    parser.add_argument('--step', type=str,metavar='123456',default='123456', help='Which steps you need.default:123456(Choose from numbers: such as \'234\')')
    parser.add_argument('--log', type=str, metavar='logfile',help='log file name,or log will print on stdout')
    parser.add_argument('--config', type=str,metavar='config.ini', help='config.ini configuration file, leave it blank to run with default parameters and the program\'s own software version.')
    parser.add_argument('--debug', action='store_true', default=False, help='The log file will contain the output of each software itself, which is convenient for finding errors (-log is required)')
    parser.add_argument('--only_script', action='store_true', default=False,help='Only generate scripts, not run automatically.')
    parser.add_argument('--cds2tree', action='store_true', default=False,help='Use cds sequence to construct gene tree.')
    args = parser.parse_args()

    if not args.log:
        coloredlogs.install(
                fmt='%(asctime)s: %(levelname)s\t%(message)s',
                evel='info'.upper(), stream=sys.stdout
            )
    else:
        if args.debug:
            l='debug'
        else:
            l='info'
        print('Logs will be written to {}'.format(args.log))
        if os.path.exists(args.log):
            os.remove(args.log)
        logging.basicConfig(
                    filename=args.log,
                    filemode='a',
                    format='%(asctime)s: %(levelname)s\t%(message)s',
                    datefmt='%H:%M:%S',
                    level=l.upper()
                    )
    home_dir=os.path.dirname(os.path.abspath(__file__))
    cf=NewConfigParser()

    if not args.config:
        logging.info('The user does not provide a configuration file, it will run with the default parameters and the software version of the program.')
        cf.add_section('software')
    else:
        cf.read(args.config)
    if not cf.has_section('software'):
        cf.add_section('software')
    abs_out=os.path.abspath(args.o)
    sp_list=[]
    logging.info("Start checking the input tree file..")
    try:
        tree_file=Phylo.read(args.tree,'newick')
        for i in tree_file.get_terminals():
            sp_list.append(i.name)
    except FileNotFoundError:
        logging.error('Checking Newicktree file :[ERR] -- The tree file :%s  does not exist' %(args.tree))
        err_exit()
    except Phylo.NewickIO.NewickError as err:
        logging.error('Checking Newicktree file :[ERR] -- %s'%(err))
        err_exit()
    logging.info("Checking Newicktree file :[OK]")
    logging.info("Start checking the input fasta file..")
    pep_postfix=cf.get("postfix", "pep") if cf.has_option("postfix", "pep") else ".pep"
    cds_postfix=cf.get("postfix", "cds") if cf.has_option("postfix", "cds") else ".cds"
    for sp in sp_list:
        seqid_list=[]
        if not os.path.exists(os.sep.join([args.i,sp+pep_postfix])):
            logging.error('The input pep file :%s does not exist.'%(os.sep.join([args.i,sp+pep_postfix])))
            err_exit()
        for s in read_fasta_file(os.sep.join([args.i,sp+pep_postfix])):
            seqid,seq = s.name,s.seq
            if '.' in s.seq:
                logging.error("The %s sequence of %s contains the character \".\", diamond will not run normally, please replace it with the \"*\" character."%(seqid,sp+pep_postfix))
                err_exit()
            seqid_list.append(s.name)
        logging.info("Checking pep file %s:[OK]"%(sp+pep_postfix))
        if  '5' in args.step or args.cds2tree:
            logging.info("Need to perform kaks calculation or use cds for tree structure, start to check the correspondence between cds file %s and pep :"%(sp+cds_postfix))
            for s in read_fasta_file(os.sep.join([args.i,sp+cds_postfix])):
                seqid,seq = s.name,s.seq
                if s.name not in seqid_list:
                    logging.error("The %s sequence id in cdsfile:%s  does not have corresponding sequence in pep file."%(s.name,sp+cds_postfix))
                    err_exit()
            logging.info("Checking cds file %s:[OK]"%(sp+cds_postfix))
    logging.info("Checking all input file :[OK]")

    logging.info("Start checking required software..")
    if '1' in args.step:
        diamond=cf.get("software", "diamond") if cf.has_option("software", "diamond") and cf.get("software", "diamond")!="" else os.sep.join([home_dir,"tree2gd","software", "diamond"])
        seqkit=cf.get("software", "seqkit") if cf.has_option("software", "seqkit") and cf.get("software", "seqkit")!="" else os.sep.join([home_dir,"tree2gd","software", "seqkit"])
        cmd_check([diamond,"help"])
        cmd_check(seqkit)
    if '2' in args.step:
        phymcl=cf.get("software", "phymcl") if cf.has_option("software", "phymcl") and cf.get("software", "phymcl")!="" else os.sep.join([home_dir,"tree2gd","software", "PhyloMCL"])
        cmd_check(phymcl)
    if '3' in args.step:
        dolloparsimony=cf.get("software", "dolloparsimony") if cf.has_option("software", "dolloparsimony") and cf.get("software", "dolloparsimony")!=""  else os.sep.join([home_dir,"tree2gd","software", "dolloparsimony"])
        cmd_check(dolloparsimony)
    if '4' in args.step:
        tree2gd=cf.get("software", "tree2gd") if cf.has_option("software", "tree2gd") and cf.get("software", "tree2gd")!="" else os.sep.join([home_dir,"tree2gd","software", "Tree2GD"])
        cmd_check(tree2gd)
        cf.set("software", "tree2gd",tree2gd)
        muscle=cf.get("software", "muscle") if cf.has_option("software", "muscle") and cf.get("software", "muscle")!="" else os.sep.join([home_dir,"tree2gd","software", "muscle"])
        cmd_check([muscle,"-version"])
        cf.set("software", "muscle",muscle)
        iqtree=cf.get("software", "iqtree") if cf.has_option("software", "iqtree") and cf.get("software", "iqtree")!="" else os.sep.join([home_dir,"tree2gd","software", "iqtree"])
        cmd_check([iqtree,"-h"])
        cf.set("software","iqtree",iqtree)
        if args.cds2tree:
            Epal2nal=cf.get("software", "Epal2nal") if cf.has_option("software", "Epal2nal") and cf.get("software", "Epal2nal")!="" else os.sep.join([home_dir,"tree2gd","software", "Epal2nal.pl"])
            cmd_check(Epal2nal)
            cf.set("software","Epal2nal",Epal2nal)
    if '5' in args.step:
        KaKs_Calculator=cf.get("software", "KaKs_Calculator") if cf.has_option("software", "KaKs_Calculator") and cf.get("software", "KaKs_Calculator")!="" else os.sep.join([home_dir,"tree2gd","software", "KaKs_Calculator"])
        cmd_check(KaKs_Calculator)
        cf.set("software", "KaKs_Calculator",KaKs_Calculator)
        muscle=cf.get("software", "muscle") if cf.has_option("software", "muscle") and cf.get("software", "muscle")!="" else os.sep.join([home_dir,"tree2gd","software", "muscle"])
        cmd_check([muscle,"-version"])
        cf.set("software", "muscle",muscle)
        Epal2nal=cf.get("software", "Epal2nal") if cf.has_option("software", "Epal2nal") and cf.get("software", "Epal2nal")!="" else os.sep.join([home_dir,"tree2gd","software", "Epal2nal.pl"])
        cmd_check(Epal2nal)
        cf.set("software", "Epal2nal",Epal2nal)


    if not os.path.exists(abs_out):
        logging.info('The output dir :%s does not exist,will create it.'%(abs_out))
        os.mkdir(abs_out)

    if '1' in args.step:
        step1out=os.sep.join([abs_out,'step1.blastp'])
        if not os.path.exists(step1out):
            logging.info('The Step1 output dir :%s does not exist,will create it.'%(step1out))
        else:
            logging.warning('The Step1 output dir :%s has exist,will backup it as %s_backup and create new.'%(step1out,step1out))
            os.rename(step1out,step1out+"_backup")
        os.mkdir(step1out)
        if not os.path.exists(os.sep.join([step1out,'sample_sh'])):
            logging.info('The Step1 scripts dir :%s does not exist,will create it.'%(os.sep.join([step1out,'sample_sh'])))
            os.mkdir(os.sep.join([step1out,'sample_sh']))
        if not os.path.exists(os.sep.join([step1out,'allsample_blast'])):
            logging.info('The Step1 blastp result dir :%s does not exist,will create it.'%(os.sep.join([step1out,'allsample_blast'])))
            os.mkdir(os.sep.join([step1out,'allsample_blast']))

        from tree2gd.blastp import all2all_diamond
        all2all_diamond(sp_list,cf,step1out,diamond,seqkit,pep_postfix,args)
        logging.info('pep all2all diamond done.')

    if '2' in args.step:
        step1out=os.sep.join([abs_out,'step1.blastp'])
        step2out=os.sep.join([abs_out,'step2.MCL'])
        if not os.path.exists(step2out):
            logging.info('The Step2 output dir :%s does not exist,will create it.'%(step2out))
        else:
            logging.warning('The Step2 output dir :%s has exist,will backup it as %s_backup and create new.'%(step2out,step2out))
            os.rename(step2out,step2out+"_backup")
        os.mkdir(step2out)
        if not cf.has_section('phymcl'):
            cf.add_section('phymcl')
        with open(args.tree) as tree_in,open(os.sep.join([step2out,"phymcl.input.tree"]),"w") as out:
            tree_text=tree_in.readlines()[0].strip('\n')
            pattern = re.compile(r":\b[0-9]+(?:\.[0-9]+)?\b")
            out.write(pattern.sub("",tree_text))
        from tree2gd.mcl import run_phymcl
        run_phymcl(step1out,phymcl,args,cf.items('phymcl'),step2out)


    if '3' in args.step:
        step1out=os.sep.join([abs_out,'step1.blastp'])
        step2out=os.sep.join([abs_out,'step2.MCL'])
        step3out=os.sep.join([abs_out,'step3.dollop'])
        if not os.path.exists(step3out):
            logging.info('The Step3 output dir :%s does not exist,will create it.'%(step3out))
        else:
            logging.warning('The Step3 output dir :%s has exist,will backup it as %s_backup and create new.'%(step3out,step3out))
            os.rename(step3out,step3out+"_backup")
        os.mkdir(step3out)
        from tree2gd.dollop import run_dolloparsimony
        run_dolloparsimony(step1out,step2out,step3out,dolloparsimony,args)

    if '4' in args.step:
        step1out=os.sep.join([abs_out,'step1.blastp'])
        step2out=os.sep.join([abs_out,'step2.MCL'])
        step4out=os.sep.join([abs_out,'step4.WGD'])

        if not os.path.exists(step4out):
            logging.info('The Step4 output dir :%s does not exist,will create it.'%(step4out))
        else:
            logging.warning('The Step4 output dir :%s has exist,will backup it as %s_backup and create new.'%(step4out,step4out))
            os.rename(step4out,step4out+"_backup")
        os.mkdir(step4out)
        if args.cds2tree:
            for i in sp_list:
                cds_path = os.sep.join([args.i,i+cds_postfix])
                with open(cds_path, 'r') as sf, open(os.sep.join([step4out,"all_cds.fa"]), 'a+') as df:
                    lines=sf.readlines()
                    for line in lines:
                        df.write(line.strip().split(" ")[0]+"\n")
        from tree2gd.wgd import run_tree2gd
        run_tree2gd(step1out,args,cf,step2out,step4out)
        shutil.rmtree(os.sep.join([step4out,'all_fa']))
        shutil.rmtree(os.sep.join([step4out,'muscle_out']))

    if '5' in args.step:
        step1out=os.sep.join([abs_out,'step1.blastp'])
        step4out=os.sep.join([abs_out,'step4.WGD'])
        step5out=os.sep.join([abs_out,'step5.KaKs'])
        gene_pairs_idmap=[]
        if not os.path.exists(step5out):
            logging.info('The Step5 output dir :%s does not exist,will create it.'%(step5out))
        else:
            logging.warning('The Step5 output dir :%s has exist,will backup it as %s_backup and create new.'%(step5out,step5out))
            os.rename(step5out,step5out+"_backup")
        os.mkdir(step5out)
        cds_postfix=cf.get("postfix", "cds") if cf.has_option("postfix", "cds") else ".cds"
        with open(os.sep.join([step1out,'all_sample.pep.faa']), 'r') as pep,open(os.sep.join([step5out,"all_pep.fa"]), 'a+') as pep_out:
            lines=pep.readlines()
            for line in lines:
                pep_out.write(line.strip().split(" ")[0]+"\n")
        for i in sp_list:
            cds_path = os.sep.join([args.i,i+cds_postfix])
            with open(cds_path, 'r') as sf, open(os.sep.join([step5out,"all_cds.fa"]), 'a+') as df:
                lines=sf.readlines()
                for line in lines:
                    df.write(line.strip().split(" ")[0]+"\n")
        with open(os.sep.join([step4out,'Tree2GD_out','gd.gene_pairs.txt']),"r") as gene_pairs,open(os.sep.join([step5out,"all_gene_pairs.list"]), 'a+') as out:
            for l in gene_pairs:
                if not "#" in l:
                    out.write(l.strip().split("\t")[4]+"\t"+l.strip().split("\t")[5]+"\n")
                    gene_pairs_idmap.append([l.strip().split("\t")[2],l.strip().split("\t")[3],l.strip().split("\t")[4],l.strip().split("\t")[5]])
        from tree2gd.kaks import run_kaks
        run_kaks(sp_list,step1out,args,cf,step4out,step5out,gene_pairs_idmap)

    if '6' in args.step:
        step2out=os.sep.join([abs_out,'step2.MCL'])
        step3out=os.sep.join([abs_out,'step3.dollop'])
        step4out=os.sep.join([abs_out,'step4.WGD'])
        step5out=os.sep.join([abs_out,'step5.KaKs'])
        step6out=os.sep.join([abs_out,'step6.plot_summary'])
        if not os.path.exists(step6out):
            logging.info('The Step6 output dir :%s does not exist,will create it.'%(step6out))
        else:
            logging.warning('The Step6 output dir :%s has exist,will backup it as %s_backup and create new.'%(step6out,step6out))
            os.rename(step6out,step6out+"_backup")
        os.mkdir(step6out)
        shutil.copyfile(os.sep.join([step2out,"allmcl.out.OGs.tabular"]),os.sep.join([step6out,"allmcl.out.OGs.tabular"]))
        shutil.copyfile(os.sep.join([step3out,"dollop.out.scv"]),os.sep.join([step6out,"dollop.out.scv"]))
        os.mkdir(os.sep.join([step6out,"Tree2GD_out"]))
        shutil.copyfile(os.sep.join([step4out,"Tree2GD_out","gd.gene_pairs.txt"]),os.sep.join([step6out,"Tree2GD_out","gd.gene_pairs.txt"]))
        shutil.copyfile(os.sep.join([step4out,"Tree2GD_out","GDtype_stat.txt"]),os.sep.join([step6out,"Tree2GD_out","GDtype_stat.txt"]))
        shutil.copyfile(os.sep.join([step4out,"Tree2GD_out","Phtree.nwk"]),os.sep.join([step6out,"Tree2GD_out","Phtree.nwk"]))
        shutil.copyfile(os.sep.join([step4out,"Tree2GD_out","summarytable.txt"]),os.sep.join([step6out,"Tree2GD_out","summarytable.txt"]))
        shutil.copytree(os.sep.join([step5out,"sp_kaks_out"]),os.sep.join([step6out,"sp_kaks_out"]))
        shutil.copyfile(os.sep.join([home_dir,"tree2gd","software","Tree2GD_draw.R"]),os.sep.join([step6out,"Tree2GD_draw.R"]))
        from tree2gd.plot import run_plot
        run_plot(step6out,args,cf)
        os.remove(os.sep.join([step6out,"Rplots.pdf"]))


def test():
    home_dir=os.path.dirname(os.path.abspath(__file__))
    parser = argparse.ArgumentParser(description='test cmd for Tree2GD')
    parser.add_argument('-t', type=int, default='1', help='Thread num.default:1')
    parser.add_argument('--config',default=os.sep.join([home_dir,"tree2gd","example_data","config.ini"]),type=str, help='config.ini configuration file, leave it blank to run with default parameters and the program\'s own software version.')
    args = parser.parse_args()
    subprocess.run("Tree2gd -i "+os.sep.join([home_dir,"tree2gd","example_data"])+" -tree "+os.sep.join([home_dir,"tree2gd","example_data","example.tree"])+" --config "+args.config+" -o ./Tree2gd_test_out -t "+str(args.t),shell=True)
    shutil.copy(os.sep.join([home_dir,"tree2gd","example_data","full_config.ini"]),"./full_config.ini")

if __name__ == '__main__':
    main()
